unit FrmMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls;

type
  TMainForm = class(TForm)
    PageControl1: TPageControl;
    TabSheet2: TTabSheet;
    CmdCloseAll: TButton;
    TabSheet1: TTabSheet;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    CmdDisplayIndex: TButton;
    ComboKeywords: TComboBox;
    CmdDisplayToc: TButton;
    ComboUrls: TComboBox;
    CmdDisplaySearch: TButton;
    CmdDisplayTopic: TButton;
    CmdDisplayTextPopup: TButton;
    ComboUrls2: TComboBox;
    RbPopupText: TRadioButton;
    EdiPopupText: TEdit;
    RbPopupId: TRadioButton;
    Label1: TLabel;
    CmdHelpContext: TButton;
    ComboHelpContext: TComboBox;
    ComboPopupId: TComboBox;
    CmdALinkLookup: TButton;
    ComboAlinkLookup: TComboBox;
    RgLookupFailure: TRadioGroup;
    CmdKeywordLookup: TButton;
    EdiKeywordLookup: TEdit;
    EdiMsgBoxText: TEdit;
    CmdGetLastError: TButton;
    hr: TLabel;
    EdiLastErrorHr: TEdit;
    Label2: TLabel;
    EdiLastErrorDescription: TEdit;
    Label3: TLabel;
    procedure CmdDisplayTextPopupClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure CmdDisplayTopicClick(Sender: TObject);
    procedure CmdDisplayIndexClick(Sender: TObject);
    procedure CmdDisplayTocClick(Sender: TObject);
    procedure CmdDisplaySearchClick(Sender: TObject);
    procedure CmdCloseAllClick(Sender: TObject);
    procedure CmdGetLastErrorClick(Sender: TObject);
    procedure CmdHelpContextClick(Sender: TObject);
    procedure CmdALinkLookupClick(Sender: TObject);
    procedure CmdKeywordLookupClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    function ApplicationOnHelp(Command: Word; Data: Longint; var CallHelp: Boolean): Boolean;
  public
  end;

var
  MainForm: TMainForm;

implementation

uses
  ActiveX, HtmlHlp;

const
  CompiledHelpFile = '..\..\Help\JediHtmlHelp.chm';

{$R *.DFM}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  // hook the help system so we can redirect to HTML Help
  Application.OnHelp := ApplicationOnHelp;
  ComboPopupId.ItemIndex := 0;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  // unhooking is not strictly necesssary but it can't hurt either
  Application.OnHelp := nil;
end;

// Miscell. tab

procedure TMainForm.CmdCloseAllClick(Sender: TObject);
begin
  HtmlHelp(0, nil, HH_CLOSE_ALL, 0);
end;

procedure TMainForm.CmdGetLastErrorClick(Sender: TObject);
var
  LastError: THHLastError;
begin
  FillChar(LastError, SizeOf(LastError), 0);
  LastError.cbStruct := SizeOf(LastError);
  EdiLastErrorHr.Text := '';
  EdiLastErrorDescription.Text := '';
  if HtmlHelp(0, PChar(CompiledHelpFile), HH_GET_LAST_ERROR, DWORD(@LastError)) <> 0 then
  begin
    EdiLastErrorHr.Text := IntToStr(LastError.hr);
    if Failed(LastError.hr) then
    begin
      if LastError.Description <> nil then
      begin
        EdiLastErrorDescription.Text := LastError.Description;
        SysFreeString(LastError.Description);
      end;
    end;
  end;
end;

// Keyword lookups

procedure TMainForm.CmdALinkLookupClick(Sender: TObject);
var
  Link: THHAKLink;
begin
  FillChar(Link, SizeOf(Link), 0);
  Link.cbStruct := SizeOf(Link);
  Link.fReserved := False;
  Link.pszKeywords := PChar(ComboALinkLookup.Text);
  Link.pszUrl := nil;
  Link.pszMsgText := nil;
  Link.pszMsgTitle := nil;
  Link.pszWindow := PChar('Main');
  Link.fIndexOnFail := False;

  case RgLookupFailure.ItemIndex of
    0: // display message box
    begin
      Link.pszMsgText := PChar(EdiMsgBoxText.Text);
      Link.pszMsgTitle := PChar('Lookup failure');
    end;
    1: // display index
    begin
      Link.fIndexOnFail := True;
    end;
    2: // display introduction.txt
    begin
      Link.pszUrl := PChar('HTML/Introduction.htm');
    end;
  end;

  HtmlHelp(0, PChar(CompiledHelpFile), HH_ALINK_LOOKUP, DWORD(@Link));
end;

procedure TMainForm.CmdKeywordLookupClick(Sender: TObject);
var
  Link: THHAKLink;
begin
  FillChar(Link, SizeOf(Link), 0);
  Link.cbStruct := SizeOf(Link);
  Link.fReserved := False;
  Link.pszKeywords := PChar(EdiKeywordLookup.Text);
  Link.pszUrl := nil;
  Link.pszMsgText := nil;
  Link.pszMsgTitle := nil;
  Link.pszWindow := PChar('Main');
  Link.fIndexOnFail := False;
  case RgLookupFailure.ItemIndex of
    0: // display message box
    begin
      Link.pszMsgText := PChar(EdiMsgBoxText.Text);
      Link.pszMsgTitle := PChar('Lookup failure');
    end;
    1: // display index
    begin
      Link.fIndexOnFail := True;
    end;
    2: // display introduction.txt
    begin
      Link.pszUrl := PChar('HTML/Introduction.htm');
    end;
  end;
  HtmlHelp(0, PChar(CompiledHelpFile), HH_ALINK_LOOKUP, DWORD(@Link));
end;

// Context-sensitive tab

procedure TMainForm.CmdDisplayTopicClick(Sender: TObject);
var
  Topic: string;
begin
  Topic := CompiledHelpFile + '::/'+ ComboUrls2.Items[ComboUrls2.ItemIndex];
  HtmlHelp(0, PChar(Topic), HH_DISPLAY_TOPIC, 0);
end;

procedure TMainForm.CmdDisplayTextPopupClick(Sender: TObject);
var
  Popup: THHPopup;
  Pt: TPoint;
begin
  FillChar(Popup, SizeOf(Popup), 0);
  Popup.cbStruct := SizeOf(Popup);
  Popup.hinst := 0;
  Popup.idString := 0;
  Popup.pszText := nil;
  GetCursorPos(Pt);
  Popup.pt := Pt;
  Popup.clrForeGround := TColorRef(-1);
  Popup.clrBackground := TColorRef(-1);
  Popup.rcMargins := Rect(-1, -1, -1, -1);
  Popup.pszFont := '';
  if RbPopupText.Checked then
  begin
    Popup.pszText := PChar(EdiPopupText.Text);
    HtmlHelp(0, PChar(CompiledHelpFile), HH_DISPLAY_TEXT_POPUP, DWORD(@Popup));
  end
  else if RbPopupId.Checked then
  begin
    Popup.idString := ComboPopupId.ItemIndex + 1;
    HtmlHelp(0, PChar(CompiledHelpFile + '::/cshelp.txt'), HH_DISPLAY_TEXT_POPUP, DWORD(@Popup));
  end;
end;

procedure TMainForm.CmdHelpContextClick(Sender: TObject);
var
  ID: Integer;
begin
  if ComboHelpContext.Text <> '' then
  begin
    try
      ID := StrToInt(ComboHelpContext.Text);
      HtmlHelp(0, PChar(CompiledHelpFile), HH_HELP_CONTEXT, ID);
    except
      on EConvertError do ShowMessage('Need a context ID in the combobox!');
    end;
  end
  else ShowMessage('Need a context ID in the combobox!');
end;

// Navigation tab

procedure TMainForm.CmdDisplayIndexClick(Sender: TObject);
begin
  HtmlHelp(0, PChar(CompiledHelpFile), HH_DISPLAY_INDEX, DWORD(PChar(ComboKeywords.Text)));
end;

procedure TMainForm.CmdDisplayTocClick(Sender: TObject);
var
  URL: string;
begin
  HtmlHelp(0, nil, HH_CLOSE_ALL, 0);
  URL := CompiledHelpFile;
  if ComboUrls.Text <> '' then URL := URL + '::/' + ComboUrls.Text;
  HtmlHelp(0, PChar(URL), HH_DISPLAY_TOC, 0);
end;

procedure TMainForm.CmdDisplaySearchClick(Sender: TObject);
var
  Qry: THHFtsQuery;
begin
  FillChar(Qry, SizeOf(Qry), 0);
  Qry.cbStruct := SizeOf(Qry);
  Qry.fUniCodeStrings := False;
  Qry.pszSearchQuery := nil;
  Qry.iProximity := HH_FTS_DEFAULT_PROXIMITY;
  Qry.fStemmedSearch := False;
  Qry.fTitleOnly := False;
  Qry.fExecute := False;
  Qry.pszWindow := nil;
  HtmlHelp(0, PChar(CompiledHelpFile), HH_DISPLAY_SEARCH, DWORD(@Qry));
end;

// Help

// This demo provides context sensitive help for all controls. Normally you would
// author and compile the topics into a helpfile or at least a resource. Unfortunately
// I am lazy by nature and choose to hardcode them as constants. I still use the
// the HH_DISPLAY_TEXT_POPUP command to show the topics though, which goes to
// show how flexible HTML Help is.

const
  HelpStrings: array[1..24] of string =
  (
    // Misccel tab
    'Closes all help windows created by this appplication',
    'Retrieves error information about the last HtmlHelp() call',
    'The numerical error code (HRESULT)',
    'A text description of the error code',
    // Keyword lookups tab
    'Performs an ALink lookup',
    'Performs an Keyword lookup',
    'List of all known alinks, choose one...',
    'Type a keyword and press the button (alle command constants are valid keywords',
    'Select the action to perform when the lookup was unsuccesfull',
    'The text to display in the messagebox when the lookup failed',
    // Context sensitive tab
    'Displays a topic from the helpfile using an HH URL',
    'Displays a text popup, like the one you''re looking at',
    'Displays help using a numerical context id',
    'The HH URL to display using HH_DISPLAY_TOPIC',
    'Use the text from the editbox below as the text for the popup',
    'This text is used for the popup',
    'Use a numeric id which links to a topic in the helpfile to display the popup',
    'All known popup id''s',
    'All known help context id''s',
    // Navigation tab
    'Displays the index tab of the helpfile',
    'Displays the Table Of Contents of the helpfile',
    'Displays the search tab of the helpfile',
    'The keyword to select in the index editbox',
    'The URL of the topic to display in the TOC'
  );

procedure ShowHelp(Pt: TPoint; ContextId: Integer);
var
  Popup: THHPopup;
begin
  // We only know about helpcontext 1 to 24
  if (ContextId >= 1) and (ContextId <= 24) then
  begin
    FillChar(Popup, SizeOf(Popup), 0);
    Popup.cbStruct := SizeOf(Popup);
    Popup.hinst := 0;
    Popup.idString := 0;
    Popup.pszText := PChar(HelpStrings[ContextId]);
    Popup.pt := Pt;
    Popup.clrForeGround := TColorRef(-1);
    Popup.clrBackground := TColorRef(-1);
    Popup.rcMargins := Rect(-1, -1, -1, -1);
    Popup.pszFont := '';
    HtmlHelp(0, nil, HH_DISPLAY_TEXT_POPUP, DWORD(@Popup));
  end;
end;

function TMainForm.ApplicationOnHelp(Command: Word; Data: Integer; var CallHelp: Boolean): Boolean;
const
  // A static local variable to remember the mouse pos
  Pos: TPoint = (X:0; Y:0);
begin
  // Instruct Delphi not to call WinHelp
  CallHelp := False;
  // Assume we can succeed
  Result := True;
  case Command of
    // Remember the position of the mouse when help was requested
    HELP_SETPOPUP_POS: Pos := SmallPointToPoint(TSmallPoint(Data));
    // This is our chance to show help. The Data parameter is the helpcontext
    // property of the control beneath the mouse cursor when help was requested,
    // either through F1 or What's This Help.
    HELP_CONTEXTPOPUP: ShowHelp(Pos, Data);
    else Result := False;
  end;
end;

end.
