unit FrmMain;

interface

// Demonstrated embedding an HTML Help window inside your form. See JediHtmlHelp.chm
// and the comments in this file for more information.

uses
  Windows, Forms, StdCtrls, Controls, Classes, ExtCtrls;

type
  TMainForm = class(TForm)
    HHEmbedPanel: TPanel;
    Panel1: TPanel;
    TopicsListBox: TListBox;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure HHEmbedPanelResize(Sender: TObject);
    procedure TopicsListBoxClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    // Handle to the embedded window, used for resizing
    FHelpWinHandle: HWND;
    procedure CreateWindow;
    procedure FillTopicsListBox;
  end;

var
  MainForm: TMainForm;

implementation

uses
  HtmlHlp;

{$R *.DFM}

const
  // The name used for the window definition
  WindowName = 'HHEmbeddedWindow';

procedure TMainForm.FormCreate(Sender: TObject);
begin
  // Fill the topics listbox and create the embedded HTML Help window
  FillTopicsListBox;
  CreateWindow;
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  // Select and display the first topic
  TopicsListBox.ItemIndex := 0;
  TopicsListBoxClick(nil);
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  // Make shure the embedded help window is closed before we exit
  HtmlHelp(0, nil, HH_CLOSE_ALL, 0);
end;

procedure TMainForm.FillTopicsListBox;
begin
  // Add a few topics to the listbox
  TopicsListBox.Items.Add('HH_UNINITIALIZE.htm');
  TopicsListBox.Items.Add('HH_ALINK_LOOKUP.htm');
  TopicsListBox.Items.Add('HH_CLOSE_ALL.htm');
  TopicsListBox.Items.Add('HH_DISPLAY_INDEX.htm');
  TopicsListBox.Items.Add('HH_DISPLAY_SEARCH.htm');
  TopicsListBox.Items.Add('HH_DISPLAY_TEXT_POPUP.htm');
  TopicsListBox.Items.Add('HH_DISPLAY_TOC.htm');
  TopicsListBox.Items.Add('HH_DISPLAY_TOPIC.htm');
  TopicsListBox.Items.Add('HH_GET_LAST_ERROR.htm');
  TopicsListBox.Items.Add('HH_GET_WIN_HANDLE.htm');
  TopicsListBox.Items.Add('HH_GET_WIN_TYPE.htm');
  TopicsListBox.Items.Add('HH_HELP_CONTEXT.htm');
  TopicsListBox.Items.Add('HH_INITIALIZE.htm');
  TopicsListBox.Items.Add('HH_KEYWORD_LOOKUP.htm');
  TopicsListBox.Items.Add('HH_PRETRANSLATEMESSAGE.htm');
  TopicsListBox.Items.Add('HH_SET_WIN_TYPE.htm');
  TopicsListBox.Items.Add('HH_SYNCH.htm');
  TopicsListBox.Items.Add('HH_TP_HELP_CONTEXTMENU.htm');
  TopicsListBox.Items.Add('HH_TP_HELP_WM_HELP.htm');
end;

procedure TMainForm.CreateWindow;
var
  WindowDef: THHWinType;
begin
  // Create simple HTML Help Window used for embedding. You can define it as you
  // want but this is about the simplest one possible.
  FillChar(WindowDef, SizeOf(WindowDef), 0);
  with WindowDef do
  begin
    cbStruct := SizeOf(WindowDef);
    // Plain ascii
    fUniCodeStrings := False;
    // Window definition name
    pszType := PChar(WindowName);
    fsValidMembers := HHWIN_PARAM_PROPERTIES or HHWIN_PARAM_STYLES or
      HHWIN_PARAM_EXSTYLES or HHWIN_PARAM_RECT or HHWIN_PARAM_SHOWSTATE or
      HHWIN_PARAM_TB_FLAGS;
    fsWinProperties := HHWIN_PROP_NOTITLEBAR or HHWIN_PROP_NODEF_STYLES or
      HHWIN_PROP_NODEF_EXSTYLES or HHWIN_PROP_TRI_PANE;
    // No caption (embedded window does not have a caption)
    pszCaption := nil;
    // The WS_CHILDWINDOW style is rather important!
    dwStyles := WS_VISIBLE or WS_CHILDWINDOW;
    // Don't need them
    dwExStyles := 0;
    // Initial size of the window -> entire client area of the panel
    rcWindowPos := Rect(0, 0, HHEmbedPanel.ClientWidth, HHEmbedPanel.ClientHeight);
    // Initially visible
    nShowState := SW_SHOW;
    // The buttons that the embedded window shows
    fsToolBarFlags := HHWIN_BUTTON_BACK or HHWIN_BUTTON_FORWARD or
      HHWIN_BUTTON_STOP or HHWIN_BUTTON_REFRESH or HHWIN_BUTTON_HOME or
      HHWIN_BUTTON_PRINT or HHWIN_BUTTON_ZOOM;
    fNotExpanded := True;
  end;
  // Create the window definition
  HtmlHelp(0, nil, HH_SET_WIN_TYPE, DWORD(@WindowDef));
end;

procedure TMainForm.HHEmbedPanelResize(Sender: TObject);
const
  SWP_UPDATEHHRECT = SWP_NOMOVE or SWP_NOACTIVATE or SWP_NOZORDER or SWP_SHOWWINDOW;
begin
  // If the MainForm is resized then resize the embedded window as well
  if IsWindow(FHelpWinHandle) then
    SetWindowPos(FHelpWinHandle, 0, 0, 0, HHEmbedPanel.ClientWidth,
      HHEmbedPanel.ClientHeight, SWP_UPDATEHHRECT);
end;

procedure TMainForm.TopicsListBoxClick(Sender: TObject);
var
  URL: string;
begin
  // Show the selected topic. The first topic displayed will cause the actual
  // window to be created, up until then there is only a definition but no
  // actual window (not that the first topic is shown in the OnShow event)
  // Also note that you have to include the window definition name in the URL
  // otherwise HTML Help will open the topic in a default (non-embedded) window.
  URL := '..\..\Help\JediHtmlHelp.chm::/HTML\Api Reference\Commands\' +
    TopicsListBox.Items[TopicsListBox.ItemIndex] + '>' + WindowName;
  // Save the window handle, we need it in the PanelResize event to resize the
  // embedded help window as well.
  FHelpWinHandle := HtmlHelp(HHEmbedPanel.Handle, PChar(URL), HH_DISPLAY_TOPIC, 0);
end;

end.
