unit FrmMain;

interface

// Demonstrates enabling and handling HTML Help notification messages. For more
// information see JediHtmlHelp.chm and the comments in this file.

uses
  Windows, Forms, Messages, Classes, Controls, StdCtrls;

type
  TMainForm = class(TForm)
    GoCommand: TButton;
    NotificationsMemo: TMemo;
    UrlCombo: TComboBox;
    procedure FormCreate(Sender: TObject);
    procedure GoCommandClick(Sender: TObject);
  private
    procedure WMNotify(var Msg: TWMNotify); message WM_NOTIFY;
  public
  end;

var
  MainForm: TMainForm;

implementation

uses
  Dialogs, HtmlHlp;

{$R *.DFM}

const
  // The used helpfile
  CompiledHelpFile = '..\..\Help\JediHtmlHelp.chm';
  // The ID that we instruct HTML Help to use when sending us notification messages
  HH_NOTIFICATION_ID = WM_USER;
  // The window name of the new, notifications enables, window type
  WindowTypeName = 'Main';

{ TMainForm }

function ActionToString(Action: Integer): string;
begin
  // Convert the numeric Action parameter to it's string representation
  // Note that our help does not display all the toolbar buttons and tabs so
  // this case statement is larger then it needed to be
  case Action of
    HHACT_TAB_CONTENTS  : Result := 'HHACT_TAB_CONTENTS';
    HHACT_TAB_INDEX     : Result := 'HHACT_TAB_INDEX';
    HHACT_TAB_SEARCH    : Result := 'HHACT_TAB_SEARCH';
    HHACT_TAB_HISTORY   : Result := 'HHACT_TAB_HISTORY';
    HHACT_TAB_FAVORITES : Result := 'HHACT_TAB_FAVORITES';
    HHACT_EXPAND        : Result := 'HHACT_EXPAND';
    HHACT_CONTRACT      : Result := 'HHACT_CONTRACT';
    HHACT_BACK          : Result := 'HHACT_BACK';
    HHACT_FORWARD       : Result := 'HHACT_FORWARD';
    HHACT_STOP          : Result := 'HHACT_STOP';
    HHACT_REFRESH       : Result := 'HHACT_REFRESH';
    HHACT_HOME          : Result := 'HHACT_HOME';
    HHACT_SYNC          : Result := 'HHACT_SYNC';
    HHACT_OPTIONS       : Result := 'HHACT_OPTIONS';
    HHACT_PRINT         : Result := 'HHACT_PRINT';
    HHACT_HIGHLIGHT     : Result := 'HHACT_HIGHLIGHT';
    HHACT_CUSTOMIZE     : Result := 'HHACT_CUSTOMIZE';
    HHACT_JUMP1         : Result := 'HHACT_JUMP1';
    HHACT_JUMP2         : Result := 'HHACT_JUMP2';
    HHACT_ZOOM          : Result := 'HHACT_ZOOM';
    HHACT_TOC_NEXT      : Result := 'HHACT_TOC_NEXT';
    HHACT_TOC_PREV      : Result := 'HHACT_TOC_PREV';
    HHACT_NOTES         : Result := 'HHACT_NOTES';
  end;
end;

procedure TMainForm.WMNotify(var Msg: TWMNotify);
begin
  // HTML Help notification messages are delivered through the standard wm_notify
  // message so we first need to check if this is a HTML Help Notification message,
  // if not then exit
  if not Msg.NMHdr.idFrom = HH_NOTIFICATION_ID then Exit;
  // Handle notifications by displaying them in the memo. In a real application
  // you would probably do something more interesting with it
  with Msg do
  case NMHdr.Code of
    HHN_NAVCOMPLETE:
    begin
      NotificationsMemo.Lines.Add('Navigation Complete');
      NotificationsMemo.Lines.Add('  URL = ' + PHHNNotify(NMHdr)^.pszUrl);
    end;
    HHN_TRACK:
    begin
      NotificationsMemo.Lines.Add('Track Notification');
      NotificationsMemo.Lines.Add('  Current URL = ' + PHHNTrack(NMHdr)^.pszCurUrl);
      NotificationsMemo.Lines.Add('  Action = ' + ActionToString(PHHNTrack(NMHdr)^.idAction));
      NotificationsMemo.Lines.Add('  WindowType = ' + PHHNTrack(NMHdr)^.phhWinType^.pszType);
    end;
    HHN_WINDOW_CREATE:
    begin
      NotificationsMemo.Lines.Add('Window Creation');
      NotificationsMemo.Lines.Add('  URL = ' + PHHNNotify(NMHdr)^.pszUrl);
    end;
    else NotificationsMemo.Lines.Add('Unknown Notification Code');
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  WinType: THHWinType;
begin
  // Create a window type named 'Main' which has notification messages enabled
  // Note that this only creates a window type and not the window itself. The
  // window won't be created until you actually attempt to display a topic. It
  // is at that point that you will receive a HHN_WINDOW_CREATE notification.
  // Especially the HHWIN_PROP_TRACKING and idNotify members are important
  // because those members are what enables the notification messages.
  FillChar(WinType, SizeOf(WinType), 0);
  with WinType do
  begin
    cbStruct := SizeOf(WinType);      // Size of this structure
    fUniCodeStrings := False;         // Use ansi strings
    pszType := PChar(WindowTypeName); // The window type name ('Main')
    fsValidMembers :=                 // The members of this structure that are valid
      HHWIN_PARAM_PROPERTIES or HHWIN_PARAM_STYLES or HHWIN_PARAM_EXSTYLES or HHWIN_PARAM_TB_FLAGS;
    fsWinProperties :=                // Window properties (appearance)
      HHWIN_PROP_TRI_PANE or HHWIN_PROP_AUTO_SYNC or HHWIN_PROP_TRACKING or
      HHWIN_PROP_TAB_SEARCH or HHWIN_PROP_TAB_HISTORY or HHWIN_PROP_TAB_FAVORITES;
    fsToolbarFlags :=                 // Toolbar buttons
      HHWIN_BUTTON_EXPAND or HHWIN_BUTTON_BACK or HHWIN_BUTTON_FORWARD or HHWIN_BUTTON_STOP or
      HHWIN_BUTTON_REFRESH or HHWIN_BUTTON_SYNC or  HHWIN_BUTTON_OPTIONS or HHWIN_BUTTON_PRINT;
    pszCaption := PChar('Notifications Enabled Help Window');
    dwStyles := $B0000;               // Window styles
    dwExStyles := 0;                  // Extended window styles
    pszToc := PChar(CompiledHelpFile + '::/TOC.hhc');
    pszIndex := PChar(CompiledHelpFile + '::/Index.hhk');
    pszFile := PChar(CompiledHelpFile + '::/HTML/Introduction.htm');
    idNotify := HH_NOTIFICATION_ID;   // Notification id send with WM_NOTIFY
  end;
  HtmlHelp(0, nil, HH_SET_WIN_TYPE, DWORD(@WinType));
end;

procedure TMainForm.GoCommandClick(Sender: TObject);
var
  URL: string;
begin
  // Display selected topic in the window. This will cause two notification
  // messages to appear in the memo. First the window creation and then,
  // assuming it is a valid topic, a navigation complete notification. AFter this
  // you can play around with the helpfile and see the notifications in the memo
  // Try playing with the buttons and selecting topics from the TOC or Index!
  URL := CompiledHelpFile + '::/' + UrlCombo.Text + '>' + WindowTypeName;
  HtmlHelp(Handle, PChar(URL), HH_DISPLAY_TOPIC, 0)
end;

end.
